@@ -71,7 +71,7 @@
 #'
 #' # create a list containing the polytomous item parameters
 #' par.prm <- list(a=c(1.4, 0.6),
-#'                d=list(c(0.0, -1.9, 1.2), c(0.4, -1.1, 1.5, 0.2)))
+#'                d=list(c(-1.9, 0.0, 1.2), c(0.4, -1.1, 1.5, 0.2)))
 #'
 #' # create a numeric vector of score categories for the items
 #' cats <- c(2, 4, 2, 2, 5, 2, 2)
@@ -105,7 +105,6 @@
 #' # compute item and test information values given the theta values
 #' info(x=test_flex, theta=theta, D=1, tif=TRUE)
 #'
-#' @export
 #'
 #' @export
 info <- function(x, ...) UseMethod("info")
@@ -113,8 +112,7 @@ info <- function(x, ...) UseMethod("info")
 #' @describeIn info Default method to compute item and test information functions for
 #' a data frame \code{x} containing the item metadata.
 #' @export
-#' @importFrom Rfast rowsums
-#' @importFrom stats na.exclude
+#' @importFrom Rfast colsums
 info.default <- function(x, theta, D=1, tif = TRUE, ...) {
 
   # confirm and correct all item metadata information
@@ -135,8 +133,10 @@ info.default <- function(x, theta, D=1, tif = TRUE, ...) {
   if(!is.null(idx.drm)) {
 
     # compute the item information
-    iif_drm <- info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
-                        b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3], D=D)
+    iif_drm <-
+      info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
+               b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3],
+               D=D, one.theta=FALSE, grad=FALSE, info=TRUE)$II
 
   } else {
     iif_drm <- NULL
@@ -145,17 +145,32 @@ info.default <- function(x, theta, D=1, tif = TRUE, ...) {
   # For PRM items
   if(!is.null(idx.prm)) {
 
-    # count the number of PRM items
-    n.prm <- length(idx.prm)
-    iif_prm <- vector('list', n.prm)
+    # count the number of examinees
+    nstd <- length(theta)
+
+    # check what poly models were used
+    pr.mod <- unique(elm_item$model[idx.prm])
+    iif_prm <- NULL
+    idx.prm <- c()
+    for(mod in pr.mod) {
+
+      # extract the response, model, and item parameters
+      lg.prm <- elm_item$model == mod
+      par.tmp <- elm_item$par[lg.prm, , drop = FALSE]
+      a <- par.tmp[, 1]
+      d <- par.tmp[, -1, drop = FALSE]
+
+      # reorder the index of the PRMs
+      idx.prm <- c(idx.prm, elm_item$item[[mod]])
+
+      # compute the probabilities of endorsing each score category
+      iif_all <-
+        info_prm(theta=theta, a=a, d=d, D=D, pr.model=mod,
+                 grad=FALSE, info=TRUE)$II
+      iif_all <- matrix(iif_all, ncol = nstd, byrow = FALSE)
+      iif_prm <- rbind(iif_prm, iif_all)
 
-    # compute the item information
-    for(k in 1:n.prm) {
-      par.tmp <- stats::na.exclude(elm_item$par[idx.prm[k], ])
-      iif_prm[[k]] <- info_prm(theta=theta, a=par.tmp[1], d=par.tmp[-1],
-                               D=D, pr.model=elm_item$model[idx.prm[k]])
     }
-    iif_prm <- do.call(what='rbind', iif_prm)
 
   } else {
     iif_prm <- NULL
@@ -184,10 +199,10 @@ info.default <- function(x, theta, D=1, tif = TRUE, ...) {
 
 }
 
+
 #' @describeIn info An object created by the function \code{\link{est_item}}.
 #' @export
-#' @importFrom Rfast rowsums
-#' @importFrom stats na.exclude
+#' @importFrom Rfast colsums
 info.est_item <- function(x, theta, tif = TRUE, ...) {
 
   # extract information from an object
@@ -212,8 +227,10 @@ info.est_item <- function(x, theta, tif = TRUE, ...) {
   if(!is.null(idx.drm)) {
 
     # compute the item information
-    iif_drm <- info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
-                        b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3], D=D)
+    iif_drm <-
+      info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
+               b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3],
+               D=D, one.theta=FALSE, grad=FALSE, info=TRUE)$II
 
   } else {
     iif_drm <- NULL
@@ -222,17 +239,32 @@ info.est_item <- function(x, theta, tif = TRUE, ...) {
   # For PRM items
   if(!is.null(idx.prm)) {
 
-    # count the number of PRM items
-    n.prm <- length(idx.prm)
-    iif_prm <- vector('list', n.prm)
+    # count the number of examinees
+    nstd <- length(theta)
+
+    # check what poly models were used
+    pr.mod <- unique(elm_item$model[idx.prm])
+    iif_prm <- NULL
+    idx.prm <- c()
+    for(mod in pr.mod) {
+
+      # extract the response, model, and item parameters
+      lg.prm <- elm_item$model == mod
+      par.tmp <- elm_item$par[lg.prm, , drop = FALSE]
+      a <- par.tmp[, 1]
+      d <- par.tmp[, -1, drop = FALSE]
+
+      # reorder the index of the PRMs
+      idx.prm <- c(idx.prm, elm_item$item[[mod]])
+
+      # compute the probabilities of endorsing each score category
+      iif_all <-
+        info_prm(theta=theta, a=a, d=d, D=D, pr.model=mod,
+                 grad=FALSE, info=TRUE)$II
+      iif_all <- matrix(iif_all, ncol = nstd, byrow = FALSE)
+      iif_prm <- rbind(iif_prm, iif_all)
 
-    # compute the item information
-    for(k in 1:n.prm) {
-      par.tmp <- stats::na.exclude(elm_item$par[idx.prm[k], ])
-      iif_prm[[k]] <- info_prm(theta=theta, a=par.tmp[1], d=par.tmp[-1],
-                               D=D, pr.model=elm_item$model[idx.prm[k]])
     }
-    iif_prm <- do.call(what='rbind', iif_prm)
 
   } else {
     iif_prm <- NULL
@@ -264,7 +296,6 @@ info.est_item <- function(x, theta, tif = TRUE, ...) {
 #' @describeIn info An object created by the function \code{\link{est_irt}}.
 #' @export
 #' @importFrom Rfast rowsums
-#' @importFrom stats na.exclude
 info.est_irt <- function(x, theta, tif = TRUE, ...) {
 
   # extract information from an object
@@ -289,8 +320,10 @@ info.est_irt <- function(x, theta, tif = TRUE, ...) {
   if(!is.null(idx.drm)) {
 
     # compute the item information
-    iif_drm <- info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
-                        b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3], D=D)
+    iif_drm <-
+      info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
+               b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3],
+               D=D, one.theta=FALSE, grad=FALSE, info=TRUE)$II
 
   } else {
     iif_drm <- NULL
@@ -299,17 +332,32 @@ info.est_irt <- function(x, theta, tif = TRUE, ...) {
   # For PRM items
   if(!is.null(idx.prm)) {
 
-    # count the number of PRM items
-    n.prm <- length(idx.prm)
-    iif_prm <- vector('list', n.prm)
+    # count the number of examinees
+    nstd <- length(theta)
+
+    # check what poly models were used
+    pr.mod <- unique(elm_item$model[idx.prm])
+    iif_prm <- NULL
+    idx.prm <- c()
+    for(mod in pr.mod) {
+
+      # extract the response, model, and item parameters
+      lg.prm <- elm_item$model == mod
+      par.tmp <- elm_item$par[lg.prm, , drop = FALSE]
+      a <- par.tmp[, 1]
+      d <- par.tmp[, -1, drop = FALSE]
+
+      # reorder the index of the PRMs
+      idx.prm <- c(idx.prm, elm_item$item[[mod]])
+
+      # compute the probabilities of endorsing each score category
+      iif_all <-
+        info_prm(theta=theta, a=a, d=d, D=D, pr.model=mod,
+                 grad=FALSE, info=TRUE)$II
+      iif_all <- matrix(iif_all, ncol = nstd, byrow = FALSE)
+      iif_prm <- rbind(iif_prm, iif_all)
 
-    # compute the item information
-    for(k in 1:n.prm) {
-      par.tmp <- stats::na.exclude(elm_item$par[idx.prm[k], ])
-      iif_prm[[k]] <- info_prm(theta=theta, a=par.tmp[1], d=par.tmp[-1],
-                               D=D, pr.model=elm_item$model[idx.prm[k]])
     }
-    iif_prm <- do.call(what='rbind', iif_prm)
 
   } else {
     iif_prm <- NULL
@@ -338,16 +386,20 @@ info.est_irt <- function(x, theta, tif = TRUE, ...) {
 
 }
 
-
 # a function to compute the expected fisher item information for DRM items
+# Pi(), Ji(), and li() functions of catR (Magis & Barrada, 2017) package were referred
+# to compute the first and second derivatives of P with respect to theta and
+# Ji value
 #' @importFrom Rfast Outer
-info_drm <- function(theta, a, b, g, D=1, one.theta = FALSE) {
+info_drm <- function(theta, a, b, g, D=1, one.theta = FALSE,
+                     r_i, grad = FALSE, info = TRUE, ji = FALSE) {
 
   # calculate probability of correct answers
+  Da <- D * a
   if(!one.theta) {
-    z <- (D * a) * Rfast::Outer(x = theta, y = b, oper = "-")
+    z <- Da * Rfast::Outer(x = theta, y = b, oper = "-")
   } else {
-    z <- (D * a) * (theta - b)
+    z <- Da * (theta - b)
   }
   P <- g + (1 - g) / (1 + exp(-z))
 
@@ -356,157 +408,284 @@ info_drm <- function(theta, a, b, g, D=1, one.theta = FALSE) {
   lg.lessg <- P < (g + 1e-10)
   P[lg.lessg] <- P[lg.lessg] + 1e-10
 
-  # compute the item information (Hambleton & Swaminathan, 1985, p.107)
-  II <- {((D * a)^2 * (1 - P)) / P} * ((P - g)/(1 - g))^2
+  # compute the fisher information
+  if(info) {
+
+    # compute the first and second derivatives of P wts the theta
+    # referred to Pi.R in the catR package (Magis and Barranda, 2017)
+    expz <- exp(z)
+    exp1g <- expz * (1 - g)
+    dP <- Da * exp1g / (1 + expz)^2
+
+    # compute the item information (Hambleton & Swaminathan, 1985, p.107)
+    # II <- {((D * a)^2 * (1 - P)) / P} * ((P - g)/(1 - g))^2
+    Q <- (1 - P)
+    II <- dP^2 / (P * Q)
+
+  } else {
+
+    II <- NULL
+
+  }
+
+  # compute the gradient of the negative loglikelihood (S)
+  if(grad) {
+    S <- - Da * ((P - g) * (r_i - P)) / ((1 - g) * P)
+  } else {
+    S <- NULL
+  }
+
+  # JI should be computed only for WL method
+  if(ji & info) {
+    d2P <- Da^2 * expz * (1 - expz) * (1 - g)/(1 + expz)^3
+    J <- dP * d2P/(P * Q)
+    #   d3P <- Da^3 * expz * (1 - g) * (expz^2 - 4 * expz + 1)/(1 + expz)^4
+    #   dJ <- (P * Q * (d2P^2 + dP * d3P) - dP^2 * d2P *(Q - P))/(P^2 * Q^2)
+    #   dI <- 2 * dP * d2P / P - dP^3 / P^2
+  } else {
+    J <- NULL
+  }
 
   # return
-  II
+  list(II = II, S = S, P = P, J = J)
 
 }
 
-# a function to compute the expected fisher item information for PRM items
-#' @importFrom Rfast rowsums
-info_prm <- function(theta, a, d, D=1, pr.model) {
 
+# a function to compute the expected fisher item information for PRM items
+# Pi(), Ji(), and li() functions of catR (Magis & Barrada, 2017) package were referred
+# to compute the first and second derivatives of P with respect to theta and
+# Ji value
+#' @importFrom Rfast rowsums
+#' @importFrom Rfast Outer
+info_prm <- function(theta, a, d, D=1, pr.model,
+                     r_i, grad = FALSE, info = TRUE, ji = FALSE) {
+
+  # check the number of step parameters
+  m <- ncol(d)
+
+  # GRM
   if(pr.model == "GRM") {
 
-    # count the number of d parameters
-    m <- length(d)
+    # convert the d matrix to a vector
+    d <- c(t(d))
 
-    # calculate all the probabilities greater than equal to each threshold
-    allPst <- drm(theta=theta, a=a, b=d, g=0, D=D)
-    allPst[allPst > 9999999999e-10] <- 9999999999e-10
-    allPst[allPst < 1e-10] <- 1e-10
-    allQst <- 1 - allPst[, ,drop=FALSE]
+    # calculate the probabilities greater than equal to each threshold
+    Da <- D * a
+    theta_d <- Rfast::Outer(x = theta, y = d, oper = "-")
+    z <- Da * matrix(theta_d, ncol = m, byrow = TRUE)
+    # z <- matrix(theta_d, ncol = m, byrow = TRUE)
+    allP <- 1 / (1 + exp(-z))
+    allP[is.na(allP)] <- 0 # insert 0 to NAs
 
-    # calculate category probabilities
-    P <- cbind(1, allPst) - cbind(allPst, 0)
+    # calculate the probability for endorsing each score category (P)
+    P <- cbind(1, allP) - cbind(allP, 0)
     P[P > 9999999999e-10] <- 9999999999e-10
     P[P < 1e-10] <- 1e-10
 
-    # compute the component values to obtain information
-    Da <- D * a
-    deriv_Pstth <- (Da * allPst) * allQst
-    deriv_Pth <- cbind(0, deriv_Pstth) - cbind(deriv_Pstth, 0)
-    w1 <- (1 - 2 * allPst) * deriv_Pstth
-    deriv2_Pth <- Da * (cbind(0, w1) - cbind(w1, 0))
+    # compute the fisher information
+    if(info) {
 
-    # compute the information for all score categories
-    ip <- (deriv_Pth^2 / P) - deriv2_Pth
+      # calculate the first and second derivative of Ps wts the theta
+      allQ <- 1 - allP[, ,drop=FALSE]
+      deriv_Pstth <- (Da * allP) * allQ
+      dP <- cbind(0, deriv_Pstth) - cbind(deriv_Pstth, 0)
+      w1 <- (1 - 2 * allP) * deriv_Pstth
+      d2P <- Da * (cbind(0, w1) - cbind(w1, 0))
+      # w2 <- Da * w1 * (1 - 2 * allP) - 2 * deriv_Pstth^2
+      # d3P <- Da * (cbind(0, w2) - cbind(w2, 0))
 
+      # compute the information for all score categories (IP)
+      IP <- (dP^2 / P) - d2P
+
+      # weight sum of all score category information
+      # which is the item information (II)
+      II <- Rfast::rowsums(IP)
+
+
+    } else {
+
+      II <- NULL
+
+    }
   }
 
+  # GPCM
   if(pr.model == "GPCM") {
 
     # include zero for the step parameter of the first category
-    d <- c(0, d)
+    # and convert the d matrix to a vector
+    d <- c(t(cbind(0, d)))
 
-    # check the number of step parameters
-    m <- length(d) - 1
-
-    # calculate category probabilities
+    # calculate the probability for endorsing each score category (P)
     Da <- D * a
-    theta_d <- (Rfast::Outer(x =  theta, y = d, oper = "-"))
-    z <- Da * theta_d
-    cumsum_z <- t(Rfast::colCumSums(z))
-    if(any(cumsum_z > 700)) {cumsum_z <- (cumsum_z / max(cumsum_z)) * 700}
-    if(any(cumsum_z < -700)) {cumsum_z <- -(cumsum_z / min(cumsum_z)) * 700}
+    theta_d <- Rfast::Outer(x =  theta, y = d, oper = "-")
+    z <- matrix(theta_d, nrow = m + 1, byrow = FALSE)
+    cumsum_z <- Da * t(Rfast::colCumSums(z))
+    # cumsum_z[is.na(cumsum_z)] <- 0
+    if(any(cumsum_z > 700, na.rm = TRUE)) {
+      cumsum_z <- (cumsum_z / max(cumsum_z, na.rm = TRUE)) * 700
+    }
+    if(any(cumsum_z < -700, na.rm = TRUE)) {
+      cumsum_z <- -(cumsum_z / min(cumsum_z, na.rm = TRUE)) * 700
+    }
     numer <- exp(cumsum_z) # numerator
-    denom <- Rfast::rowsums(numer)
+    numer[is.na(numer)] <- 0
+    denom <- Rfast::rowsums(numer, na.rm = TRUE)
     P <- (numer / denom)
+    P[P < 1e-10] <- 1e-10
 
-    # compute the component values to obtain information
-    denom2 <- denom^2
-    d1th_z <- matrix(Da * (1:(m+1)), nrow=length(theta), ncol=(m+1), byrow=TRUE)
-    d1th_denom <- Rfast::rowsums(numer * d1th_z)
-    deriv_Pth <- (numer / denom2) * (d1th_z * denom - d1th_denom)
-    denom4 <- denom2^2
-    d2th_denom <- Rfast::rowsums(numer * d1th_z^2)
-    d1th_z_den <- d1th_z * denom
-    part1 <- d1th_z_den * denom * (d1th_z_den - d1th_denom)
-    part2 <- denom2 * (d1th_z * d1th_denom + d2th_denom) - 2 * denom * d1th_denom^2
-    deriv2_Pth <- (numer / denom4) * (part1 - part2)
+    # compute the fisher information
+    if(info) {
 
-    # compute the information for all score categories
-    ip <- (deriv_Pth^2 / P) - deriv2_Pth
+      # calculate the first and second derivative of Ps wts the theta
+      denom2 <- denom^2
+      denom4 <- denom2^2
+      d1th_z <- (1:(m+1))
+      d1th_denom <- Da * as.numeric(tcrossprod(x = d1th_z, y = numer))
+      d2th_denom <- Da^2 * as.numeric(tcrossprod(x = d1th_z^2, y = numer))
+      d1th_z_den <- Da * outer(X =  denom, Y = d1th_z, FUN = "*")
+      dP <- (numer / denom2) * (d1th_z_den - d1th_denom)
+      part1 <- d1th_z_den * denom * (d1th_z_den - d1th_denom)
+      part2 <-
+        denom2 * (Da * outer(X =  d1th_denom, Y = d1th_z, FUN = "*") + d2th_denom) -
+        2 * denom * d1th_denom^2
+      d2P <- (numer / denom4) * (part1 - part2)
 
+      # compute the information for all score categories (IP)
+      IP <- (dP^2 / P) - d2P
+
+      # weight sum of all score category information
+      # which is the item information (II)
+      II <- Rfast::rowsums(IP)
+
+    } else {
+
+      dP <- dP2 <- II <- NULL
+
+    }
   }
 
-  # sum of all score category information
-  II <- Rfast::rowsums(ip)
+  # compute the gradient of the negative loglikelihood (S)
+  if(grad) {
+
+    frac_rp <- r_i / P
+    S <- -Rfast::rowsums(frac_rp * dP)
+
+  } else {
+    S <- NULL
+  }
+
+  # JI should be computed only for WL method
+  # and S is also adjusted accordingly using the weight function of J/2I
+  if(ji & info) {
+    J <- Rfast::rowsums((dP * d2P) / P)
+  } else {
+    J <- NULL
+  }
 
   # return the results
-  II
+  list(II = II, S = S, P = P, J = J)
 
 }
 
-
-# This function computes a negative expectation of second derivative of log-likelihood (fisher information)
-info_score <- function(theta, elm_item, idx.drm, idx.prm,
-                       method=c("ML", "MAP", "MLF"), D=1, norm.prior=c(0, 1)) {
+# This function computes the gradient of the negative loglikelihood and
+# a negative expectation of second derivative of log-likelihood (fisher information)
+info_score <- function(theta, elm_item, freq.cat, idx.drm, idx.prm,
+                       method=c("ML", "MAP", "MLF", "WL"), D=1,
+                       norm.prior=c(0, 1), grad=TRUE, ji=FALSE) {
 
   # For DRM items
   if(!is.null(idx.drm)) {
 
-    # compute the fisher information
-    finfo_drm <-
-      sum(info_drm(theta=theta, a=elm_item$pars[idx.drm, 1],
-                   b=elm_item$par[idx.drm, 2], g=elm_item$par[idx.drm, 3],
-                   D=D, one.theta=TRUE))
+    # extract the response and item parameters
+    r_i <- freq.cat[idx.drm, 2]
+    a <- elm_item$pars[idx.drm, 1]
+    b <- elm_item$pars[idx.drm, 2]
+    g <- elm_item$pars[idx.drm, 3]
+
+    # compute the gradient and fisher information
+    gi_drm <-
+      info_drm(theta=theta, a=a, b=b, g=g,
+               D=D, one.theta=TRUE, r_i = r_i, grad=grad,
+               info=TRUE, ji=ji)
+    grad_drm <- gi_drm$S
+    finfo_drm <- gi_drm$II
+    ji_drm <- gi_drm$J
 
   } else {
 
-    # assign 0 to the fisher information
+    # assign 0 to the gradient and fisher information
+    grad_drm <- 0
     finfo_drm <- 0
+    ji_drm <- 0
 
   }
 
   # For PRM items
   if(!is.null(idx.prm)) {
 
-    # count the number of the PRM items
-    n.prm <- length(idx.prm)
-
-    # compute the fisher information
+    pr.mod <- unique(elm_item$model[idx.prm])
+    grad_prm <- c()
     finfo_prm <- c()
-    for(i in 1:n.prm) {
-      par.tmp <- stats::na.exclude(elm_item$par[idx.prm[i], ])
-      finfo_prm[i] <-
-        info_prm(theta=theta, a=par.tmp[1], d=par.tmp[-1],
-                 D=D, pr.model=elm_item$model[idx.prm[i]])
-    }
+    ji_prm <- c()
+    for(mod in pr.mod) {
 
-    # sum of the information for all PRM items
-    finfo_prm <- sum(finfo_prm)
+      # extract the response, model, and item parameters
+      lg.prm <- elm_item$model == mod
+      par.tmp <- elm_item$par[lg.prm, , drop = FALSE]
+      r_i <- freq.cat[lg.prm, , drop = FALSE]
+      a <- par.tmp[, 1]
+      d <- par.tmp[, -1, drop = FALSE]
+
+      # compute the gradient and fisher information
+      gi_prm <-
+        info_prm(theta=theta, a=a, d=d, D=D, pr.model=mod,
+                 r_i=r_i, grad=grad, info=TRUE, ji=ji)
+      grad_prm <- c(grad_prm, gi_prm$S)
+      finfo_prm <- c(finfo_prm, gi_prm$II)
+      ji_prm <- c(ji_prm, gi_prm$J)
+
+    }
 
   } else {
 
-    # assign 0 to the fisher information
+    # assign 0 to the gradient and fisher information
+    grad_prm <- 0
     finfo_prm <- 0
+    ji_prm <- 0
 
   }
 
-  # sum of the fisher information for DRM and PRM items
-  finfo <- sum(finfo_drm, finfo_prm)
+  # sum of the gradients and the fisher information for DRM and PRM items
+  grad_sum <- sum(grad_drm, grad_prm)
+  finfo_sum <- sum(finfo_drm, finfo_prm)
+  if(ji) {
+    ji_sum <- sum(ji_drm, ji_prm)
+    grad_sum <- grad_sum - (ji_sum / (2 * finfo_sum))
+  }
 
   # extract the fisher information when MAP method is used
   if(method == "MAP") {
 
-    # compute a hessian of prior distribution
+    # compute a gradient and hessian of prior distribution
     rst.prior <-
       logprior_deriv(val=theta, is.aprior=FALSE, D=NULL, dist="norm",
                      par.1=norm.prior[1], par.2=norm.prior[2])
 
-    # extract the hessian
+    # extract the hessian and add it
     finfo.prior <- attributes(rst.prior)$hessian
+    finfo_sum <- sum(finfo_sum, finfo.prior)
 
-    # add the hessian
-    finfo <- sum(finfo, finfo.prior)
+    # add the gradient and add it
+    if(grad) {
+      grad.prior <- attributes(rst.prior)$gradient
+      grad_sum <- sum(grad_sum, grad.prior)
+    }
 
   }
 
   # return results
-  finfo
+  list(finfo = finfo_sum, grad = grad_sum)
 
 }
-
